package cn.wch.ch934xuartdemo;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.hardware.usb.UsbDevice;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.SwitchCompat;
import androidx.appcompat.widget.Toolbar;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.os.Handler;
import android.os.Looper;
import android.text.method.ScrollingMovementMethod;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.TextView;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import cn.wch.ch934xlib.CH934XManager;
import cn.wch.ch934xlib.callback.IUsbStateChange;
import cn.wch.ch934xlib.chip.ChipType;
import cn.wch.ch934xlib.exception.ChipException;
import cn.wch.ch934xlib.exception.NoPermissionException;
import cn.wch.ch934xlib.exception.UartLibException;
import cn.wch.ch934xuartdemo.R;
import cn.wch.ch934xuartdemo.adapter.DeviceAdapter;
import cn.wch.ch934xuartdemo.entity.DeviceEntity;
import cn.wch.ch934xuartdemo.entity.SerialEntity;
import cn.wch.ch934xuartdemo.ui.CustomTextView;
import cn.wch.ch934xuartdemo.ui.DeviceListDialog;
import cn.wch.ch934xuartdemo.ui.GPIODialog;
import cn.wch.ch934xuartdemo.utils.FormatUtil;
import cn.wch.ch934xuartdemo.utils.LogUtil;
import cn.wch.ch934xuartdemo.utils.ToastUtil;


public class MainActivity extends AppCompatActivity {
    RecyclerView deviceRecyclerVIew;
    DeviceAdapter deviceAdapter;
    private Context context;

    //接收区
    TextView readBuffer;
    CustomTextView clearRead;
    SwitchCompat scRead;
    TextView readCount;
    //保存各个串口的接收计数
    HashMap<String, Integer> readCountMap=new HashMap<>();
    //已打开的设备列表
    final Set<UsbDevice> devices= Collections.synchronizedSet(new HashSet<UsbDevice>());
    //读线程
    Thread readThread;
    boolean flag=false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.context=this;
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        if(!UsbFeatureSupported()){
            showToast("系统不支持USB Host功能");
            System.exit(0);
            return;
        }
        initUI();
        readThread=new ReadThread();
        readThread.start();

    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_maim,menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        int itemId = item.getItemId();
        if(itemId==R.id.enumDevice){
            enumDevice();
        }else if(itemId==R.id.GPIO){
            openGPIOConfigDialog();
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        //停止读线程
        stopReadThread();
        //停止文件测试
        //cancelLinks();
        //关闭所有连接设备
        closeAll();
        //释放资源
        CH934XManager.getInstance().close(this);
    }

    /**
     * 系统是否支持USB Host功能
     *
     * @return true:系统支持USB Host false:系统不支持USB Host
     */
    public boolean UsbFeatureSupported() {
        boolean bool = this.getPackageManager().hasSystemFeature(
                "android.hardware.usb.host");
        return bool;
    }

    void initUI(){
        deviceRecyclerVIew=findViewById(R.id.rvDevice);
        readBuffer=findViewById(R.id.tvReadData);
        clearRead=findViewById(R.id.tvClearRead);
        scRead=findViewById(R.id.scRead);
        readCount=findViewById(R.id.tvReadCount);
        //初始化recyclerview
        deviceRecyclerVIew.setNestedScrollingEnabled(false);
        deviceAdapter =new DeviceAdapter(this);

        deviceRecyclerVIew.setLayoutManager(new LinearLayoutManager(this,LinearLayoutManager.VERTICAL,false));
        deviceAdapter.setEmptyView(LayoutInflater.from(this).inflate(R.layout.empty_view,deviceRecyclerVIew,false));
        deviceRecyclerVIew.setAdapter(deviceAdapter);
        deviceAdapter.setActionListener(new DeviceAdapter.OnActionListener() {
            @Override
            public void onRemove(UsbDevice usbDevice) {
                removeReadDataDevice(usbDevice);
            }
        });

        readBuffer.setMovementMethod(ScrollingMovementMethod.getInstance());
        clearRead.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                clearReadData();
            }
        });
        //监测USB插拔状态
        monitorUSBState();
        //动态申请权限
        if(ContextCompat.checkSelfPermission(this,Manifest.permission.WRITE_EXTERNAL_STORAGE)!= PackageManager.PERMISSION_GRANTED){
            ActivityCompat.requestPermissions(this,new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},111);
        }

    }

    /**
     * 枚举当前所有符合要求的设备，显示设备列表
     */
    void enumDevice(){
        try {
            //枚举符合要求的设备
            ArrayList<UsbDevice> usbDeviceArrayList = CH934XManager.getInstance().enumDevice();
            if(usbDeviceArrayList.size()==0){
                showToast("no matched devices");
                return;
            }
            //显示设备列表dialog
            DeviceListDialog deviceListDialog=DeviceListDialog.newInstance(usbDeviceArrayList);
            deviceListDialog.setCancelable(false);
            deviceListDialog.show(getSupportFragmentManager(),DeviceListDialog.class.getName());
            deviceListDialog.setOnClickListener(new DeviceListDialog.OnClickListener() {
                @Override
                public void onClick(UsbDevice usbDevice) {
                    //选择了某一个设备打开
                    open(usbDevice);
                }
            });
        } catch (UartLibException e) {
            LogUtil.d(e.getMessage());
        }
    }

    /**
     * 打开GPIO配置界面
     */
    void openGPIOConfigDialog(){

        ArrayList<UsbDevice> connectedDevices = CH934XManager.getInstance().getConnectedDevices();
        if(connectedDevices.size()==0){
            showToast("no connected device!");
            return;
        }

        DeviceListDialog deviceListDialog=DeviceListDialog.newInstance(connectedDevices);
        deviceListDialog.setCancelable(false);
        deviceListDialog.show(getSupportFragmentManager(),DeviceListDialog.class.getName());
        deviceListDialog.setOnClickListener(new DeviceListDialog.OnClickListener() {
            @Override
            public void onClick(UsbDevice usbDevice) {
                //选择了某一个设备打开
                int serialCount = CH934XManager.getInstance().getSerialCount(usbDevice);
                GPIODialog dialog=GPIODialog.newInstance(usbDevice,serialCount);
                dialog.setCancelable(false);
                dialog.show(getSupportFragmentManager(),GPIODialog.class.getName());
            }
        });


    }

    /**
     * 从设备列表中打开某个设备
     *
     * @param usbDevice
     */
    void open(@NonNull UsbDevice usbDevice){
        if(CH934XManager.getInstance().isConnected(usbDevice)){
            showToast("当前设备已经打开");
            return;
        }
        try {
            boolean b = CH934XManager.getInstance().openDevice(usbDevice);
            if(b){
                //打开成功
                //更新显示的ui
                update(usbDevice);
                //初始化接收计数
                int serialCount = CH934XManager.getInstance().getSerialCount(usbDevice);
                for (int i = 0; i < serialCount; i++) {
                    readCountMap.put(FormatUtil.getSerialKey(usbDevice,i),0);
                }
                //将该设备添加至已打开设备列表,在读线程ReadThread中,将会读取该设备的每个串口数据
                addToReadDeviceSet(usbDevice);
                //用作文件对比测试,在打开每个设备时，对每个串口新建对应的保存数据的文件
//                for (int i = 0; i < serialCount; i++) {
//                    linkSerialToFile(usbDevice,i);
//                }

            }else {
                showToast("打开失败");
            }
        } catch (ChipException e) {
            LogUtil.d(e.getMessage());
        } catch (NoPermissionException e) {
            //没有权限打开该设备
            //申请权限
            showToast("没有权限打开该设备");
            requestPermission(usbDevice);
        } catch (UartLibException e) {
            e.printStackTrace();
        }
    }

    /**
     * 申请读写权限
     * @param usbDevice
     */
    private void requestPermission(@NonNull UsbDevice usbDevice){
        try {
            CH934XManager.getInstance().requestPermission(this,usbDevice);
        } catch (UartLibException e) {
            LogUtil.d(e.getMessage());
        }
    }

    /**
     * 监测USB的状态
     */
    private void monitorUSBState(){
        CH934XManager.getInstance().setUsbStateListener(new IUsbStateChange() {
            @Override
            public void usbDeviceDetach(UsbDevice device) {
                //设备移除
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        //从界面上移除
                        if(deviceAdapter!=null){
                            deviceAdapter.removeDevice(device);
                        }
                    }
                });

            }

            @Override
            public void usbDeviceAttach(UsbDevice device) {
                //设备插入
            }

            @Override
            public void usbDevicePermission(UsbDevice device, boolean result) {
                //请求打开设备权限结果
            }
        });
    }

    /**
     * //recyclerView更新UI
     * @param usbDevice
     */
    void update(UsbDevice usbDevice){
        //根据vid/pid获取芯片类型
        ChipType chipType = CH934XManager.getInstance().getChipType(usbDevice);
        //获取芯片串口数目,为负则代表出错
        int serialCount = CH934XManager.getInstance().getSerialCount(usbDevice);
        //构建recyclerView所绑定的数据,添加设备
        ArrayList<SerialEntity> serialEntities=new ArrayList<>();
        for (int i = 0; i < serialCount; i++) {
            SerialEntity serialEntity=new SerialEntity(usbDevice,i);
            serialEntities.add(serialEntity);
        }
        DeviceEntity deviceEntity=new DeviceEntity(usbDevice,chipType.getDescription(),serialEntities);
        if(deviceAdapter.hasExist(deviceEntity)){
            //已经显示
            showToast("该设备已经存在");
        }else {
            deviceAdapter.addDevice(deviceEntity);
        }
    }

    /**
     * 关闭所有设备
     */
    void closeAll(){
        ArrayList<UsbDevice> usbDeviceArrayList = null;
        try {
            usbDeviceArrayList = CH934XManager.getInstance().enumDevice();
            for (UsbDevice usbDevice : usbDeviceArrayList) {
                if(CH934XManager.getInstance().isConnected(usbDevice)){
                    CH934XManager.getInstance().disconnect(usbDevice);
                }
            }
        } catch (UartLibException e) {
            e.printStackTrace();
        }

    }

    private void addToReadDeviceSet(@NonNull UsbDevice usbDevice){
        synchronized (devices){
            devices.add(usbDevice);
        }

    }

    private void removeReadDataDevice(@NonNull UsbDevice usbDevice){
        synchronized (devices){
            devices.remove(usbDevice);
        }
    }

    public class ReadThread extends Thread{

        public ReadThread() {
            flag=true;
            setPriority(Thread.MAX_PRIORITY);
        }

        @Override
        public void run() {
            LogUtil.d("---------------开始读取数据");
            while (flag){
                if(devices.isEmpty()){
                    continue;
                }
                //遍历已打开的设备列表中的设备
                synchronized (devices){
                    Iterator<UsbDevice> iterator = devices.iterator();
                    while (iterator.hasNext()){
                        UsbDevice device = iterator.next();
                        int serialCount = CH934XManager.getInstance().getSerialCount(device);
                        //读取该设备每个串口的数据
                        for (int i = 0; i < serialCount; i++) {
                            try {
                                byte[] bytes = CH934XManager.getInstance().readData(device, i);
                                if(bytes!=null){
                                    //界面显示数据
                                    updateReadData(device,i,bytes,bytes.length);
                                    //数据保存至文件
                                    //updateReadDataToFile(device,i,bytes,bytes.length);
                                }
                            } catch (ChipException e) {
                                //LogUtil.d(e.getMessage());
                                break;
                            }
                        }
                    }
                }
            }
            LogUtil.d("读取数据线程结束");
        }
    }

    public void stopReadThread(){
        if(readThread!=null && readThread.isAlive()){
            flag=false;
        }
    }

    private void updateReadData(@NonNull UsbDevice usbDevice,int serialNumber, byte[] buffer, int length){
        if(buffer==null){
            return;
        }
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Integer integer = readCountMap.get(FormatUtil.getSerialKey(usbDevice, serialNumber));
                if(integer==null){
                    //不包含此key
                    return;
                }
                //更新计数
                integer+=length;
                readCountMap.put(FormatUtil.getSerialKey(usbDevice, serialNumber),integer);
                //

                String result="";
                if (readBuffer.getText().toString().length() >= 1500) {
                    readBuffer.setText("");
                    readBuffer.scrollTo(0, 0);
                }

                if(scRead.isChecked()){
                    result= FormatUtil.bytesToHexString(buffer, length);

                }else {
                    result=new String(buffer,0,length);
                }
                String readBufferLogPrefix = FormatUtil.getReadBufferLogPrefix(usbDevice, serialNumber,integer);
                //LogUtil.d(readBufferLogPrefix);
                readBuffer.append(readBufferLogPrefix+result+"\r\n");

                int offset = readBuffer.getLineCount() * readBuffer.getLineHeight();
                //int maxHeight = usbReadValue.getMaxHeight();
                int height = readBuffer.getHeight();
                //USBLog.d("offset: "+offset+"  maxHeight: "+maxHeight+" height: "+height);
                if (offset > height) {
                    //USBLog.d("scroll: "+(offset - usbReadValue.getHeight() + usbReadValue.getLineHeight()));
                    readBuffer.scrollTo(0, offset - readBuffer.getHeight() + readBuffer.getLineHeight());
                }
            }
        });
    }


    private void clearReadData(){
        readBuffer.scrollTo(0,0);
        readBuffer.setText("");
        for (String s : readCountMap.keySet()) {
            readCountMap.put(s,0);
        }
    }

    private void showToast(String message){
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                ToastUtil.create(context,message).show();
            }
        });
    }
    ///////////////////////////////////////将数据保存至文件,与发送文件对比测试////////////////////////////////////////////////////

    //该Map的key是每个设备的串口，value是其对应的保存数据的文件的fileStream
    private HashMap<String, FileOutputStream> fileOutputStreamMap=new HashMap<>();

    //用作文件对比测试,在打开每个设备时，每个串口都新建对应的保存数据的文件，其映射关系保存到fileOutputStreamMap中
    private void linkSerialToFile(UsbDevice usbDevice,int serialNumber){
        LogUtil.d("linkSerialToFile:");
        File testFile = getExternalFilesDir("TestFile");
        File file=new File(testFile,CH934XManager.getInstance().getChipType(usbDevice).toString()+"_"+serialNumber+".txt");
        if(file.exists()){
            file.delete();
        }
        try {
            boolean ret = file.createNewFile();
            LogUtil.d("新建文件:"+ret);
            FileOutputStream fileOutputStream = new FileOutputStream(file);
            if(!fileOutputStreamMap.containsKey(FormatUtil.getSerialKey(usbDevice,serialNumber))){
                fileOutputStreamMap.put(FormatUtil.getSerialKey(usbDevice,serialNumber),fileOutputStream);
            }
        } catch (IOException e) {
            LogUtil.d(e.getMessage());
        }

    }

    //将接收到的数据保存至文件，用作对比
    private void updateReadDataToFile(@NonNull UsbDevice usbDevice,int serialNumber, byte[] buffer, int length){
        updateToFile(usbDevice, serialNumber, buffer, length);
    }

    private void updateToFile(@NonNull UsbDevice usbDevice,int serialNumber, byte[] buffer, int length){
        if(fileOutputStreamMap.containsKey(FormatUtil.getSerialKey(usbDevice,serialNumber))){
            FileOutputStream fileOutputStream = fileOutputStreamMap.get(FormatUtil.getSerialKey(usbDevice, serialNumber));
            try {
                fileOutputStream.write(buffer,0,length);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    //结束保存至文件的功能,关闭Stream
    private void cancelLinks(){
        for (String s : fileOutputStreamMap.keySet()) {
            FileOutputStream fileOutputStream = fileOutputStreamMap.get(s);
            try {
                fileOutputStream.flush();
                fileOutputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}