package cn.wch.ch934xuartdemo.ui;

import android.hardware.usb.UsbDevice;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.RadioGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;

import com.touchmcu.ui.DialogUtil;

import cn.wch.ch934xlib.CH934XManager;
import cn.wch.ch934xlib.chip.Mode;
import cn.wch.ch934xlib.exception.UartLibException;
import cn.wch.ch934xlib.gpio.GPIO_DIR;
import cn.wch.ch934xlib.gpio.GPIO_VALUE;
import cn.wch.ch934xuartdemo.R;
import cn.wch.ch934xuartdemo.databinding.DialogGpioBinding;
import cn.wch.ch934xuartdemo.utils.LogUtil;
import cn.wch.ch934xuartdemo.utils.ToastUtil;
import io.reactivex.Observable;
import io.reactivex.ObservableEmitter;
import io.reactivex.ObservableOnSubscribe;
import io.reactivex.Observer;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

public class GPIODialog extends DialogFragment {

    private DialogGpioBinding binding;

    private int serialCount=4;
    private UsbDevice device;
    private Handler handler=new Handler(Looper.getMainLooper());

    public static GPIODialog newInstance(UsbDevice device,int serialCount) {
        Bundle args = new Bundle();
        GPIODialog fragment = new GPIODialog(serialCount, device);
        fragment.setArguments(args);
        return fragment;
    }

    public GPIODialog(int serialCount, UsbDevice device) {
        this.serialCount = serialCount;
        this.device = device;
    }

    @Override
    public void onResume() {
        super.onResume();
        getDialog().getWindow().setLayout(WindowManager.LayoutParams.MATCH_PARENT,
                WindowManager.LayoutParams.WRAP_CONTENT);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        getDialog().requestWindowFeature(Window.FEATURE_NO_TITLE);
        binding = DialogGpioBinding.inflate(inflater, container, false);
        init();
        return binding.getRoot();
    }




    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding=null;
    }

    private void init() {
        //init serial spinner
        String[] items=new String[serialCount];
        for (int i = 0; i < items.length; i++) {
            items[i]=Integer.toString(i);
        }
        ArrayAdapter<String> arrayAdapter=new ArrayAdapter<String>(getContext(), R.layout.my_spinner_textview);
        arrayAdapter.addAll(items);

        arrayAdapter.setDropDownViewResource(R.layout.my_spinner_textview);
        binding.spSerial.setAdapter(arrayAdapter);
        binding.spSerial.setGravity(0x10);
        binding.spSerial.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                LogUtil.d("spinner selected: "+position);
                refreshMode();
                refreshDir();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
        binding.spSerial.setSelection(0);
        //get current mode
        binding.tvGetCurrentMode.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                refreshMode();
            }
        });
        //enable mode
        binding.tvEnableMode.setOnMyClickListener(new MyTextView.OnMyClickListener() {
            @Override
            public void onEnableGPIO() {
                int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
                enableGPIO(device,serial,true);
            }

            @Override
            public void onDisEnableGPIO() {
                int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
                enableGPIO(device,serial,false);
            }
        });



        binding.tvClose.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });

        refreshDir();

        initGPIOValues();
    }



    //invoke when serial change
    private void refreshMode(){
        int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
        getCurrentMode(serial);
    }

    private void refreshDir(){
        cancelDirs();
        try {
            Thread.sleep(100);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
//        for(int i=serial*3;i<serial*3+3;i++){
//            try {
//                GPIO_DIR gpioDir = CH934XManager.getInstance().getGPIODirFromCache(device, i);
//                if(i==serial*3){
//                    if(gpioDir==GPIO_DIR.DIR_IN){
//                        binding.rg0.checkWithCallback(binding.rbIn0.getId());
//
//                    }else {
//                        binding.rg0.checkWithCallback(binding.rbOut0.getId());
//                    }
//
//                }else if(i==serial*3+1){
//                    if(gpioDir==GPIO_DIR.DIR_IN){
//                        binding.rg1.checkWithCallback(binding.rbIn1.getId());
//                    }else {
//                        binding.rg1.checkWithCallback(binding.rbOut1.getId());
//                    }
//
//                }else if(i==serial*3+2){
//                    if(gpioDir==GPIO_DIR.DIR_IN){
//                        binding.rg2.checkWithCallback(binding.rbIn2.getId());
//                    }else {
//                        binding.rg2.checkWithCallback(binding.rbOut2.getId());
//                    }
//                }
//
//            } catch (UartLibException e) {
//                showToast(e.getMessage());
//                return;
//            }
//        }
        binding.rg0.clearCheck();
        binding.rg1.clearCheck();
        binding.rg2.clearCheck();
        try {
            Thread.sleep(100);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        initGPIODirs();



    }

    private void cancelDirs(){
        binding.rg0.setNewListener(null);
        binding.rg1.setNewListener(null);
        binding.rg2.setNewListener(null);

    }

    private void initGPIODirs() {
        binding.tvGetDir.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                refreshDir();
                showToast("success!");
            }
        });

        binding.rg0.setNewListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
                if(checkedId==binding.rbIn0.getId()){
                    setGPIODir(device,serial*3,GPIO_DIR.IN);
                }else if(checkedId==binding.rbOut0.getId()){
                    setGPIODir(device,serial*3,GPIO_DIR.OUT);
                }
            }
        });
        binding.rg1.setNewListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
                if(checkedId==binding.rbIn1.getId()){
                    setGPIODir(device,serial*3+1,GPIO_DIR.IN);
                }else if(checkedId==binding.rbOut1.getId()){
                    setGPIODir(device,serial*3+1,GPIO_DIR.OUT);
                }
            }
        });
        binding.rg2.setNewListener(new RadioGroup.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(RadioGroup group, int checkedId) {
                int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
                if(checkedId==binding.rbIn2.getId()){
                    setGPIODir(device,serial*3+2,GPIO_DIR.IN);
                }else if(checkedId==binding.rbOut2.getId()){
                    setGPIODir(device,serial*3+2,GPIO_DIR.OUT);
                }
            }
        });

    }

    private void initGPIOValues(){
        binding.cbValue0.setMonitorListener(new MyCheckBox.MonitorState() {
            @Override
            public void onWishStatus(boolean checked) {
                int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
                setGPIOValue(device,serial*3,checked?GPIO_VALUE.HIGH :GPIO_VALUE.LOW,binding.cbValue0);
            }
        });
        binding.cbValue1.setMonitorListener(new MyCheckBox.MonitorState() {
            @Override
            public void onWishStatus(boolean checked) {
                int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
                setGPIOValue(device,serial*3+1,checked?GPIO_VALUE.HIGH :GPIO_VALUE.LOW,binding.cbValue1);
            }
        });
        binding.cbValue2.setMonitorListener(new MyCheckBox.MonitorState() {
            @Override
            public void onWishStatus(boolean checked) {
                int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
                setGPIOValue(device,serial*3+2,checked?GPIO_VALUE.HIGH :GPIO_VALUE.LOW,binding.cbValue2);
            }
        });

        binding.tvGetPin.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int serial = Integer.parseInt((String) binding.spSerial.getSelectedItem());
                getGPIOValue(device,serial);
            }
        });

    }



    private void getCurrentMode(int serialNumber){
        try {
            Mode currentMode = CH934XManager.getInstance().getCurrentMode(device, serialNumber);
            if(currentMode!=null){
                binding.tvCurrentMode.setText(currentMode.getDes());
                if(currentMode==Mode.HARDFLOW){
                    showToast("close flow first!");
                    binding.tvEnableMode.setVisibility(View.INVISIBLE);
                    return;
                }
                binding.tvEnableMode.setCurrentMode(currentMode);
            }
        } catch (UartLibException e) {
            showToast(e.getMessage());
        }
    }

    private void enableGPIO(UsbDevice device,int serialNumber,boolean enabled){
        Observable.create(new ObservableOnSubscribe<String>() {
            @Override
            public void subscribe(@io.reactivex.annotations.NonNull ObservableEmitter<String> emitter) throws Exception {

                try {
                    boolean ret = CH934XManager.getInstance().enableGPIO(device, serialNumber, enabled);
                    if(!ret){
                        emitter.onError(new Throwable("operation failed!"));
                    }else {
                        emitter.onComplete();
                        return;
                    }
                } catch (UartLibException e) {
                    emitter.onError(new Throwable(e.getMessage()));
                }

            }
        })
                .subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Observer<String>() {
                    @Override
                    public void onSubscribe(@io.reactivex.annotations.NonNull Disposable d) {
                        DialogUtil.getInstance().showLoadingDialog(getActivity(),"");
                    }

                    @Override
                    public void onNext(@io.reactivex.annotations.NonNull String s) {

                    }

                    @Override
                    public void onError(@io.reactivex.annotations.NonNull Throwable e) {
                        DialogUtil.getInstance().hideLoadingDialog();
                        showToast(e.getMessage());
                    }

                    @Override
                    public void onComplete() {
                        DialogUtil.getInstance().hideLoadingDialog();
                        getCurrentMode(serialNumber);
                        refreshDir();
                    }
                });
    }

    private void setGPIODir(UsbDevice device, int gpioNumber, GPIO_DIR dir){
        LogUtil.d("setGPIODir: "+dir);
        Observable.create(new ObservableOnSubscribe<String>() {
            @Override
            public void subscribe(@io.reactivex.annotations.NonNull ObservableEmitter<String> emitter) throws Exception {

                try {
                    boolean ret = CH934XManager.getInstance().setGPIODir(device, gpioNumber, dir);
                    if(!ret){
                        emitter.onError(new Throwable("operation failed!"));
                    }else {
                        emitter.onComplete();
                        return;
                    }
                } catch (UartLibException e) {
                    emitter.onError(new Throwable(e.getMessage()));
                }

            }
        })
                .subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Observer<String>() {
                    @Override
                    public void onSubscribe(@io.reactivex.annotations.NonNull Disposable d) {
                        DialogUtil.getInstance().showLoadingDialog(getActivity(),"");
                    }

                    @Override
                    public void onNext(@io.reactivex.annotations.NonNull String s) {

                    }

                    @Override
                    public void onError(@io.reactivex.annotations.NonNull Throwable e) {
                        DialogUtil.getInstance().hideLoadingDialog();

                        showToast(e.getMessage());

                    }

                    @Override
                    public void onComplete() {
                        DialogUtil.getInstance().hideLoadingDialog();
                        showToast("operation success!");

                    }
                });
    }

    private void setGPIOValue(UsbDevice device, int gpioNumber, GPIO_VALUE value,CheckBox checkBox){
        LogUtil.d("setGPIOValue: "+gpioNumber+" "+value);
        Observable.create(new ObservableOnSubscribe<String>() {
            @Override
            public void subscribe(@io.reactivex.annotations.NonNull ObservableEmitter<String> emitter) throws Exception {

                try {
                    boolean ret = CH934XManager.getInstance().setGPIOValue(device,gpioNumber,value);
                    if(!ret){
                        emitter.onError(new Throwable("operation failed!"));
                    }else {
                        emitter.onComplete();
                        return;
                    }
                } catch (UartLibException e) {
                    emitter.onError(new Throwable(e.getMessage()));
                }

            }
        })
                .subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Observer<String>() {
                    @Override
                    public void onSubscribe(@io.reactivex.annotations.NonNull Disposable d) {
                        DialogUtil.getInstance().showLoadingDialog(getActivity(),"");
                    }

                    @Override
                    public void onNext(@io.reactivex.annotations.NonNull String s) {

                    }

                    @Override
                    public void onError(@io.reactivex.annotations.NonNull Throwable e) {
                        DialogUtil.getInstance().hideLoadingDialog();

                        showToast(e.getMessage());
                        checkBox.setChecked(value != GPIO_VALUE.HIGH);
                    }

                    @Override
                    public void onComplete() {
                        DialogUtil.getInstance().hideLoadingDialog();
                        showToast("operation success!");
                        checkBox.setChecked(value == GPIO_VALUE.HIGH);
                    }
                });
    }

    private void getGPIOValue(UsbDevice device,int serialNumber){
        LogUtil.d("getGPIOValue: "+serialNumber);
        Observable.create(new ObservableOnSubscribe<String>() {
            @Override
            public void subscribe(@io.reactivex.annotations.NonNull ObservableEmitter<String> emitter) throws Exception {

                try {
                    boolean ret = CH934XManager.getInstance().getGPIOValue(device);
                    if(!ret){
                        emitter.onError(new Throwable("get GPIO value fail!"));
                        return;
                    }
                    for (int i=serialNumber*3;i<serialNumber*3+3;i++){
                        GPIO_VALUE gpio_value = CH934XManager.getInstance().queryGPIOValueFromCache(device, i);
                        if(i==serialNumber*3){
                            handler.post(new Runnable() {
                                @Override
                                public void run() {
                                    binding.cbValue0.setChecked(gpio_value==GPIO_VALUE.HIGH);
                                }
                            });
                        }else if(i==serialNumber*3+1){
                            handler.post(new Runnable() {
                                @Override
                                public void run() {
                                    binding.cbValue1.setChecked(gpio_value==GPIO_VALUE.HIGH);
                                }
                            });
                        }else if(i==serialNumber*3+2){
                            handler.post(new Runnable() {
                                @Override
                                public void run() {
                                    binding.cbValue2.setChecked(gpio_value==GPIO_VALUE.HIGH);
                                }
                            });
                        }
                    }
                    emitter.onComplete();
                }catch (UartLibException e){
                    emitter.onError(new Throwable(e.getMessage()));
                    return;
                }

            }
        })
                .subscribeOn(Schedulers.newThread())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Observer<String>() {
                    @Override
                    public void onSubscribe(@io.reactivex.annotations.NonNull Disposable d) {
                        DialogUtil.getInstance().showLoadingDialog(getActivity(),"");
                    }

                    @Override
                    public void onNext(@io.reactivex.annotations.NonNull String s) {
                        showToast(s);
                    }

                    @Override
                    public void onError(@io.reactivex.annotations.NonNull Throwable e) {
                        DialogUtil.getInstance().hideLoadingDialog();

                        showToast(e.getMessage());
                    }

                    @Override
                    public void onComplete() {
                        DialogUtil.getInstance().hideLoadingDialog();
                        //showToast("operation success!");
                    }
                });

    }

    private void showToast(String message){
        handler.post(new Runnable() {
            @Override
            public void run() {
                ToastUtil.create(getContext(),message).show();
            }
        });
    }
}
